@@ -48,7 +48,8 @@ eps = 1e-5  # used for finite difference derivative calculations
 class ModtranRT(TabularRT):
     """A model of photon transport including the atmosphere."""
 
-    def __init__(self, engine_config: RadiativeTransferEngineConfig, full_config: Config):
+    def __init__(self, engine_config: RadiativeTransferEngineConfig, full_config: Config, 
+                 build_lut: bool = True):
         """."""
 
         # Specify which of the potential MODTRAN LUT parameters are angular, which will be handled differently
@@ -98,7 +99,9 @@ class ModtranRT(TabularRT):
         self.last_point_lookup_values = np.zeros(self.n_point)
 
         # Build the lookup table
-        self.build_lut()
+        if build_lut:
+            self.build_lut()
+
 
     def find_basedir(self, config):
         """Seek out a modtran base directory."""
@@ -220,6 +223,23 @@ class ModtranRT(TabularRT):
         if hasattr(self, 'aer_absc'):
             fracs = np.zeros((self.naer))
 
+        if 'IPARM' not in param[0]['MODTRANINPUT']['GEOMETRY']:
+            raise AttributeError('MODTRAN template requires an IPARM specification')
+
+        if param[0]['MODTRANINPUT']['GEOMETRY']['ITYPE'] != 3:
+            raise AttributeError('Currently unsupported modtran ITYPE specification')
+
+        # Geometry values that depend on IPARM
+        if param[0]['MODTRANINPUT']['GEOMETRY']['IPARM'] == 12 and \
+            'GMTIME' in overrides.keys():
+            raise AttributeError('GMTIME in MODTRAN driver overrides, but IPARM set to 12.  Check modtran template.')
+        elif param[0]['MODTRANINPUT']['GEOMETRY']['IPARM'] == 11 and \
+            set(['solar_azimuth','solaz','solar_zenith','solzen']).intersection(set(overrides.keys())):
+            raise AttributeError('Solar geometry (solar az/azimuth zen/zenith) is specified, but IPARM is set to 12.  Check MODTRAN template') 
+        
+        if set(['PARM1','PARM2']).intersection(set(overrides.keys())):
+            raise AttributeError('PARM1 and PARM2 keys not supported as LUT dimensions.  Please use either solar_azimuth/solaz or solar_zenith/solzen')
+
         # Perform overrides
         for key, val in overrides.items():
             recursive_replace(param, key, val)
@@ -228,17 +248,38 @@ class ModtranRT(TabularRT):
                 i = int(key.split('_')[-1])
                 fracs[i] = val
 
-            elif key == 'EXT550' or key == 'AOT550' or key == 'AOD550':
+            elif key in ['EXT550', 'AOT550', 'AOD550']:
                 # MODTRAN 6.0 convention treats negative visibility as AOT550
                 recursive_replace(param, 'VIS', -val)
 
             elif key == 'FILTNM':
                 param[0]['MODTRANINPUT']['SPECTRAL']['FILTNM'] = val
-
-            elif key in ['ITYPE', 'H1ALT', 'IDAY', 'IPARM', 'PARM1',
-                         'PARM2', 'GMTIME', 'TRUEAZ', 'OBSZEN']:
+            
+            # Geometry parameters we want to populate even if unassigned
+            elif key in ['H1ALT', 'IDAY', 'TRUEAZ','OBSZEN', 'GMTIME' ]:
                 param[0]['MODTRANINPUT']['GEOMETRY'][key] = val
 
+            # Surface parameters we want to populate even if unassigned
+            elif key in ['GNDALT']:
+                param[0]['MODTRANINPUT']['SURFACE'][key] = val
+
+            elif key in ['solar_azimuth', 'solaz']:
+                if 'TRUEAZ' not in param[0]['MODTRANINPUT']['GEOMETRY']:
+                    raise AttributeError('Cannot have solar azimuth in LUT without specifying TRUEAZ.  Use RELAZ instead.')
+                param[0]['MODTRANINPUT']['GEOMETRY']['PARM1'] = param[0]['MODTRANINPUT']['GEOMETRY']['TRUEAZ'] - val + 180
+
+            elif key in ['solar_zenith', 'solzen']:
+                param[0]['MODTRANINPUT']['GEOMETRY']['PARM2'] = abs(val)
+            
+            #elif key in ['altitude_km']
+
+            elif key in ['DISALB', 'NAME']:
+                recursive_replace(param, key, val)
+            elif key in param[0]['MODTRANINPUT']['ATMOSPHERE'].keys():
+                recursive_replace(param, key, val)
+            else:
+                raise AttributeError('Unsupported MODTRAN parameter {} specified'.format(key))
+
         # For custom aerosols, specify final extinction and absorption
         # MODTRAN 6.0 convention treats negative visibility as AOT550
         if hasattr(self, 'aer_absc'):
@@ -297,15 +338,13 @@ class ModtranRT(TabularRT):
                     cmd = self.rebuild_cmd(point, filebase)
 
                     # Run MODTRAN for up to 10 seconds - this should be plenty of time
-                    cwd = os.getcwd()
                     if os.path.isdir(self.lut_dir) is False:
                         os.mkdir(self.lut_dir)
-                    os.chdir(self.lut_dir)
                     try:
-                        subprocess.call(cmd, shell=True, timeout=10)
+                        subprocess.call(cmd, shell=True, timeout=10,
+                                        cwd=self.lut_dir)
                     except:
                         pass
-                    os.chdir(cwd)
 
 
                     max_water = None
@@ -333,6 +372,7 @@ class ModtranRT(TabularRT):
 
         self.wl = mod_outputs[0]['wl']
         self.solar_irr = mod_outputs[0]['sol']
+        np.save('solar_irr.npy',self.solar_irr)
         self.coszen = np.cos(mod_outputs[0]['solzen'] * np.pi / 180.0)
 
         dims_aug = self.lut_dims + [self.n_chan]
@@ -470,7 +510,9 @@ class ModtranRT(TabularRT):
             elif name == "OBSZEN":
                 point[point_ind] = geom.OBSZEN
             elif name == "GNDALT":
-                point[point_ind] = geom.GNDALT
+                point[point_ind] = geom.surface_elevation_km
+            elif name == "H1ALT":
+                point[point_ind] = geom.observer_altitude_km
             elif name == "viewzen":
                 point[point_ind] = geom.observer_zenith
             elif name == "viewaz":
